#ifndef FERI_INTEGRATION
#define FERI_INTEGRATION
// -----------------------------------------------
// Computation of B-Series
//
// Integration

#include "main.hpp"
#include "tools.hpp"
#include "tree_structures.hpp"

// -----------------------------------------------
// integrating
template<typename VectorFieldType, typename CoefficientsType>
VectorType integrate(VectorFieldType &F, VectorType &x, GraphType &DAG, TreeList &tree_list, CoefficientsType &alpha)
{
	VectorType result(x.size());
	VertexIterator DAG_it;
	std::pair<VertexIterator, VertexIterator> GRAPHlimits;
	ColorMap color = get(boost::vertex_color, DAG);
	typedef TreeList::iterator TreeListIterator;
	TreeListIterator tree_it;
	long int i = 0;

	// -----------------------------------------------
	// processing the DAG [it is already represented in post-order]
	#ifdef FERI_DEBUG
		std::cout << "evaluating the DAG..." << std::flush;
	#endif

	evaluate_graph(F, x, DAG, tree_list);

	#ifdef FERI_DEBUG
		std::cout << " done" << std::endl << std::flush;
	#endif

	// -----------------------------------------------
	// summing the result
	#ifdef FERI_DEBUG
		std::cout << "summing the values..." << std::flush;
	#endif

	GRAPHlimits = boost::vertices(DAG);

	result = x;

	for(tree_it = tree_list.begin(); tree_it != tree_list.end(); ++tree_it)
	{
		++i;
		result = result + alpha(i) * get(color, (*tree_it).vertex);
	}

	#ifdef FERI_DEBUG
		std::cout << " done" << std::endl << std::flush;
	#endif

	return result;
}

// -----------------------------------------------
// testing
template<typename VectorFieldType, typename CoefficientsType>
void test(VectorFieldType &F, VectorType &x, GraphType &DAG, TreeList &tree_list, CoefficientsType &alpha)
{
	VectorType result(x.size());
	VertexIterator DAG_it;
	std::pair<VertexIterator, VertexIterator> GRAPHlimits;
	ColorMap color = get(boost::vertex_color, DAG);
	typedef TreeList::iterator TreeListIterator;
	TreeListIterator tree_it;
	long int i = 0;

	// -----------------------------------------------
	// processing the DAG [it is already represented in post-order]
	#ifdef FERI_DEBUG
		std::cout << "evaluating the DAG..." << std::flush;
	#endif

	evaluate_graph(F, x, DAG, tree_list);

	#ifdef FERI_DEBUG
		std::cout << " done" << std::endl << std::flush;
	#endif

	// -----------------------------------------------
	// summing the result
	#ifdef FERI_DEBUG
		std::cout << "Summing the values..." << std::endl << std::flush;
	#endif

	GRAPHlimits = boost::vertices(DAG);

	int last_size = 1;
	for(tree_it = tree_list.begin(); tree_it != tree_list.end(); ++tree_it)
	{
		if ((*tree_it).tree.size() > last_size) {
			std::cout << "Sum for trees with " << last_size << " vertices:" << std::endl << result << std::endl << std::endl;
			result.clear();
		}
		++i;
		std::cout << "tau: " << (*tree_it).tree << std::endl;
		std::cout << "F_f(tau): " << get(color, (*tree_it).vertex) << std::endl;
		std::cout << "alpha(tau): " << alpha(i) << std::endl;
		result = result + alpha(i) * get(color, (*tree_it).vertex);
		last_size = (*tree_it).tree.size();
	}
	std::cout << "Sum for trees with " << last_size << " vertices:" << std::endl << result << std::endl;

	#ifdef FERI_DEBUG
		std::cout << "done." << std::endl << std::flush;
	#endif
}
// ~ end of file ~
#endif
