#ifndef FERI_MAIN
#define FERI_MAIN
// -----------------------------------------------
// Computation of B-Series
//
// Main Header

// -----------------------------------------------
// includes

// std
#include <iostream>
#include <fstream>
#include <cstdio>
#include <iomanip>
#include <typeinfo>
#include <string>
#include <cassert>
#include <ctime>
#include <cstdlib>

// capd
#include <capd/dynsys/FadMap.h>
#include <capd/fadbad/multiprec_fadbad.h>
#include <capd/vectalg/Vector.hpp>
#include <capd/vectalg/Matrix.hpp>
#include <capd/dynsys/BasicFadTaylor.h>

// boost
#include <boost/config.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/graph_traits.hpp>
#include <boost/property_map/dynamic_property_map.hpp>
#include <boost/graph/graphviz.hpp>

// mpfr
#include "capd/multiPrec/MpReal.h"

// -----------------------------------------------

// DEBUG MODE
#define FERI_DEBUG
//#define FERI_DEBUG_PARC
//#define FERI_DEBUG_DAG

// MULTI PRECISION MODE
//#define FERI_MPFR

// -----------------------------------------------
// typedefs

	// scalars
	#ifdef FERI_MPFR
		typedef capd::multiPrec::MpReal ScalarType;
		const int MPrecision = 200;
	#else
		typedef double ScalarType;
	#endif

	// Matrices & Vectors
	typedef capd::vectalg::Matrix<ScalarType, 0, 0> SeedMatrixType;
	typedef capd::vectalg::Vector<ScalarType, 0> MultiIndexType;
	typedef capd::vectalg::Vector<int, 0> IndexType;
	typedef capd::vectalg::Vector<ScalarType, 0> VectorType;

	// graphs
	typedef boost::property < boost::vertex_name_t, std::string, boost::property < boost::vertex_color_t, VectorType > > VertexType;
	typedef boost::property < boost::edge_weight_t, int > EdgeType;

	typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::directedS, VertexType, EdgeType>	GraphType;

	typedef boost::property_map<GraphType, boost::vertex_color_t>::type ColorMap;
	typedef boost::property_map<GraphType, boost::vertex_name_t>::type NameMap;
	typedef boost::property_map<GraphType, boost::edge_weight_t>::type EdgeWeightMap;
	typedef boost::property_map<GraphType, boost::vertex_index_t>::type IndexMap;

	typedef GraphType::vertex_iterator		VertexIterator;
	typedef GraphType::out_edge_iterator	OutEdgeIterator;
	typedef GraphType::vertex_descriptor	VertexDescriptor;
	typedef GraphType::edge_descriptor 		EdgeDescriptor;
	typedef std::pair<EdgeDescriptor, bool> EdgeAccess;

	typedef std::list<VertexDescriptor>		VertexList;
	typedef VertexList::iterator			VertexListIterator;

// --------------------------------------------

// ~ end of file ~
#endif
