#ifndef FERI_VECTORFIELD
#define FERI_VECTORFIELD
// -----------------------------------------------
// Computation of B-Series
//
// Vectorfield

// -----------------------------------------------

#include "main.hpp"

// -----------------------------------------------
// The function: R^N -> R^M
template<int N, int M>
class VectorField
{
public:
	typedef fadbad::T<ScalarType> TScalar;
	typedef capd::vectalg::Vector<TScalar, N> TInVector;
	typedef capd::vectalg::Vector<TScalar, M> TOutVector;
	typedef capd::vectalg::Vector<ScalarType, N> InVector;
	typedef capd::vectalg::Vector<ScalarType, 0> DynamicVector;
	typedef capd::vectalg::Vector<ScalarType, M> OutVector;

	// constructor
	VectorField():m_in(N){
		m_out = (*this)(m_in);
		m_t = 0;
		m_t[1] = 1;
	}

	// dimension of the function
	std::pair<int, int> dimension()
	{
		std::pair<int, int> result;
		result.first = N;
		result.second = M;

		return result;
	}

	// the function in -> out
	template<typename InVectorType, typename OutVectorType>
	void operator()(const InVectorType &in, OutVectorType &out)
	{
/*
		out[0] = exp( sin(in[0]) / cos(2 * in[1]) );
		out[1] = exp( sin(in[0]) * tan(in[1]) / cos( pow(in[0], 2) ) );
		out[2] = tan(in[2]) * pow(in[1], 2) * cos(in[0]);
*/
		out[0] = exp( sin(in[0]) / cos(2 * in[1]) ) / (log(pow(in[0], 2) * pow(in[1], 2) + 5) * in[2]);
		out[1] = exp( sin(in[0]) * tan(in[1]) / (log(pow(in[0], 2) + 1) * in[1] * in[2]) ) / (pow(in[0], 2) + in[2]);
		out[2] = tan(in[2]) * pow(in[1], 2) * cos(in[0]);
/*
		out[0] = in[0];
		out[1] = pow( in[1], 2 );
		out[2] = sin(in[0]) * in[2];
*/
/*
		out[0] = sin(in[0]) * cos(2 * in[1]);
		out[1] = sin(in[0]) * tan(in[1]) + cos( pow(in[0], 2) );
		out[2] = tan(in[2]) * pow(in[1], 2) * cos(in[0]);
*/
	}

	// individual realization for Taylor vectors
	TOutVector operator()(const TInVector &in)
	{
		TOutVector out;

		(*this)(in, out);

		return out;
	}

	// individual realization for normal vectors
	OutVector operator()(const InVector &in)
	{
		OutVector out;

		(*this)(in, out);

		return out;
	}

	// individual realization for dynamic vectors
	DynamicVector operator()(const DynamicVector &in)
	{
		DynamicVector out(M);

		(*this)(in, out);

		return out;
	}

	// evaluating the i-th Taylor coefficient in a given direction
	template<typename InVectorType>
	void evaluate_from(const InVectorType &x, const DynamicVector &s, const int &i, VectorType &result)
	{
		for (int comp = 1; comp <= M; ++comp) {
			m_out(comp).reset();

			for (int j = 0; j < N; ++j)
				m_in[j] = m_t * s[j] + x[j];

			m_out = (*this)(m_in);

			m_out(comp).eval(i);
			result(comp) = m_out(comp)[i];
		}
	}

protected:
	TOutVector m_out;
	TInVector m_in;
	TScalar m_t;
};

// for testing the program
template<class Scalar, int D>
class TestVectorField : public capd::dynsys::FadMap<Scalar,D>
{
public:
	typedef typename capd::dynsys::FadMap<Scalar,D>::MatrixType MatrixType;
	typedef typename capd::dynsys::FadMap<Scalar,D>::VectorType VectorType;

	template <typename AVector>
	AVector operator()(const AVector& in) const
	{
		AVector out(D);
		out[0] = in[0];
		out[1] = pow( in[1], 2 );
		out[2] = sin(in[0]) * in[2];
		return out;
	}

	MatrixType operator[](const VectorType& u) const
	{
		MatrixType der(D,D);
		computeDerivative(*this,u,der);
		return der;
	}


	int dimension() const {return D;}

};

// ~ end of file ~
#endif
